from .enum import Enum


class AuthPermission(Enum):

    uploadPublicKey = "auth/uploadPublicKey"


class RefundPermission(Enum):

    viewTeams = "refund/viewTeams"
    createTeams = "refund/createTeams"
    deleteTeams = "refund/deleteTeams"
    editTeams = "refund/editTeams"
    payRefunds = "refund/payRefunds"
    viewRefunds = "refund/viewRefunds"
    createRefunds = "refund/createRefunds"
    deleteRefunds = "refund/deleteRefunds"
    approveRefunds = "refund/approveRefunds"
    viewRoles = "refund/viewRoles"
    editRoles = "refund/editRoles"


class BankPermission(Enum):

    viewTransaction = "bank/viewTransaction"
    createTransaction = "bank/createTransaction"


class TransferPermission(Enum):

    viewRequests = "transfer/viewRequest"
    createRequest = "transfer/createRequest"


class ChargePaymentPermission(Enum):

    viewPayments = "chargePayment/viewPayments"
    createPayments = "chargePayment/createPayments"


class TaxPaymentPermission(Enum):

    viewPayments = "taxPayment/viewPayments"
    createPayments = "taxPayment/createPayments"


class ChargePermission(Enum):

    viewCharges = "charge/viewCharges"
    createCharges = "charge/createCharges"


class InvoicePermission(Enum):

    viewInvoices = "invoice/viewInvoices"
    createInvoices = "invoice/createInvoices"
    refundInvoices = "deposit/refundInvoices"


class DepositPermission(Enum):

    viewDeposits = "deposit/viewDeposits"
    refundDeposits = "deposit/refundDeposits"


class BrcodePaymentPermission(Enum):

    viewPayments = "brcodePayment/viewPayments"
    createPayments = "brcodePayment/createPayments"


class WorkspacePermission(Enum):

    viewRoles = "workspace/viewRoles"
    editRoles = "workspace/editRoles"


class WorkspaceRole(Enum):

    admin = "admin"
    member = "member"
    financial = "financial"
    invoiceOperator = "invoiceOperator"

    @classmethod
    def permissions(self, role):
        return {
            WorkspaceRole.invoiceOperator: (
                [
                    WorkspacePermission.viewRoles,
                    InvoicePermission.viewInvoices,
                    ChargePermission.viewCharges,
                ]
            ),
            WorkspaceRole.member: (
                [
                    RefundPermission.viewTeams,
                    RefundPermission.viewRoles,
                    WorkspacePermission.viewRoles,
                    ChargePaymentPermission.viewPayments,
                    TaxPaymentPermission.viewPayments,
                    BrcodePaymentPermission.viewPayments,
                ]
            ),
            WorkspaceRole.financial: (
                [
                    RefundPermission.viewTeams,
                    RefundPermission.viewRoles,
                    WorkspacePermission.viewRoles
                ]
                + [
                    BankPermission.viewTransaction,
                    BankPermission.createTransaction,
                ]
                + TransferPermission.values()
                + ChargePaymentPermission.values()
                + TaxPaymentPermission.values()
                + ChargePermission.values()
                + DepositPermission.values()
                + InvoicePermission.values()
                + BrcodePaymentPermission.values()
            ),
            WorkspaceRole.admin: (
                RefundPermission.values()
                + AuthPermission.values()
                + BankPermission.values()
                + TransferPermission.values()
                + ChargePaymentPermission.values()
                + TaxPaymentPermission.values()
                + ChargePermission.values()
                + WorkspacePermission.values()
                + DepositPermission.values()
                + InvoicePermission.values()
                + BrcodePaymentPermission.values()
            ),
        }.get(role, [])

    @classmethod
    def contains(cls, permission, inRoles):
        for role in inRoles:
            if permission in cls.permissions(role):
                return True
        return False
